//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
#include "VKDetectorMessenger.hh"
#include "VKDetectorConstruction.hh"

#include "G4UIcmdWithADoubleAndUnit.hh"
#include "G4UIcmdWith3VectorAndUnit.hh"
#include "G4UIcmdWithAnInteger.hh"
#include "G4UIcommand.hh"
#include "G4UIdirectory.hh"
#include "G4UIcmdWithABool.hh"
#include "G4UIcmdWithADouble.hh"
#include "G4Scintillation.hh"


VKDetectorMessenger::VKDetectorMessenger(VKDetectorConstruction* pDetConstr)
 : fDetConstr(pDetConstr)
{
    // Setup a command directory for detector controls with guidance
    fDetectorDir = new G4UIdirectory("/VK/detector/");
    fDetectorDir->SetGuidance("Detector geometry control");

    fVolumesDir = new G4UIdirectory("/VK/detector/volumes/");
    fVolumesDir->SetGuidance("Enable/disable volumes");
       
    // Various commands for modifying detector geometry
    fMagnetCmd = new G4UIcmdWithABool("/VK/detector/volumes/magnet", this);
    fMagnetCmd->SetGuidance("Enable/Disable the magnet.");
    fMagnetCmd->AvailableForStates(G4State_PreInit, G4State_Idle);
    fMagnetCmd->SetToBeBroadcasted(false);

    fBetaDetectorCmd = new G4UIcmdWithABool("/VK/detector/volumes/betaDetector", this);
    fBetaDetectorCmd->SetGuidance("Enable/Disable the beta detector.");
    fBetaDetectorCmd->AvailableForStates(G4State_PreInit, G4State_Idle);
    fBetaDetectorCmd->SetToBeBroadcasted(false);

    fCatcherFoilCmd = new G4UIcmdWithABool("/VK/detector/volumes/catcherFoil", this);
    fCatcherFoilCmd->SetGuidance("Enable/Disable the catcher foil.");
    fCatcherFoilCmd->AvailableForStates(G4State_PreInit, G4State_Idle);
    fCatcherFoilCmd->SetToBeBroadcasted(false);

    fShieldCmd = new G4UIcmdWithABool("/VK/detector/volumes/shield", this);
    fShieldCmd->SetGuidance("Enable/Disable the shield.");
    fShieldCmd->AvailableForStates(G4State_PreInit, G4State_Idle);
    fShieldCmd->SetToBeBroadcasted(false);

    fVirtualDetectorsCmd = new G4UIcmdWithABool("/VK/storeTrajectoryData", this);
    fVirtualDetectorsCmd->SetGuidance("Enable/Disable storage of trajectory data.");
    fVirtualDetectorsCmd->AvailableForStates(G4State_PreInit, G4State_Idle);
    fVirtualDetectorsCmd->SetToBeBroadcasted(false);

    fCheckOverlapsCmd = new G4UIcmdWithABool("/VK/detector/volumes/checkOverlaps", this);
    fCheckOverlapsCmd->SetGuidance("Enable/Disable check for overlaps every time a volume is (re)created.");
    fCheckOverlapsCmd->AvailableForStates(G4State_PreInit, G4State_Idle);
    fCheckOverlapsCmd->SetToBeBroadcasted(false);

    // Define /VK/field command directory
    fFieldDir = new G4UIdirectory("/VK/field/");
    fFieldDir -> SetGuidance("Magnetic field control");

    fBFieldCmd = new G4UIcmdWithABool("/VK/field/enable", this);
    fBFieldCmd -> SetGuidance("Enable/Disable the magnetic Field.");
    fBFieldCmd -> AvailableForStates(G4State_PreInit, G4State_Idle);
    fBFieldCmd -> SetToBeBroadcasted(false);

    fMaxBFieldCmd = new G4UIcmdWithADoubleAndUnit("/VK/field/maxbfield", this);
    fMaxBFieldCmd -> SetGuidance("Set maximum magnetic field strength.");
    fMaxBFieldCmd -> SetParameterName("fieldstrength", false);
    fMaxBFieldCmd -> SetDefaultUnit("tesla");
    fMaxBFieldCmd -> AvailableForStates(G4State_PreInit, G4State_Idle);
    fMaxBFieldCmd -> SetToBeBroadcasted(false);    
}


VKDetectorMessenger::~VKDetectorMessenger()
{
    delete fMagnetCmd;
    delete fBetaDetectorCmd;
    delete fCatcherFoilCmd;
    delete fShieldCmd;
    delete fVirtualDetectorsCmd;
    delete fBFieldCmd;
    delete fMaxBFieldCmd;
    delete fCheckOverlapsCmd;
}


void VKDetectorMessenger::SetNewValue(G4UIcommand* command, G4String newValue)
{
    if (command == fMagnetCmd)
    {
        fDetConstr -> SetMagnetOn(fMagnetCmd -> GetNewBoolValue(newValue));
    }
    else if (command == fBetaDetectorCmd)
    {
        fDetConstr -> SetBetaDetectorOn(fBetaDetectorCmd -> GetNewBoolValue(newValue));
    }
    else if (command == fCatcherFoilCmd)
    {
        fDetConstr -> SetCatcherFoilOn(fCatcherFoilCmd -> GetNewBoolValue(newValue));
    }
    else if (command == fShieldCmd)
    {
        fDetConstr -> SetShieldOn(fShieldCmd -> GetNewBoolValue(newValue));
    }
    else if (command == fVirtualDetectorsCmd)
    {
        fDetConstr -> SetVirtualDetectorsOn(fVirtualDetectorsCmd -> GetNewBoolValue(newValue));
    }
    else if (command == fCheckOverlapsCmd)
    {
        fDetConstr -> SetCheckOverlaps(fCheckOverlapsCmd -> GetNewBoolValue(newValue));
    }
    else if (command == fBFieldCmd)
    {
        fDetConstr -> SetBFieldOn(fBFieldCmd -> GetNewBoolValue(newValue));
    }
    else if (command == fMaxBFieldCmd)
    {
        fDetConstr -> SetMaxBField(fMaxBFieldCmd -> GetNewDoubleValue(newValue));
    }
}
