
#include "VKMaterials.hh"

#include "globals.hh"
#include "G4Material.hh"
#include "G4NistManager.hh"
#include "G4UnitsTable.hh"

/**
 * Constructor
 */
VKMaterials::VKMaterials()
{
	// NIST Material Manager
	nistMan = G4NistManager::Instance();
	nistMan -> SetVerbose(0);

	// Create materials
	CreateMaterials();
	
	// Set optical properties
	SetOpticalPropertiesEJ200();
	SetOpticalPropertiesPMMA();
	SetOpticalPropertiesTeflon();
	SetOpticalPropertiesGlass();
}

VKMaterials::~VKMaterials()
{
    // do nothing 
}


/**
 * Setting the Singleton Class
 */
VKMaterials* VKMaterials::instance=0;  


/**
 * Instance of the Singleton Class
 * @return the instance of the Singleton Materials
 */
VKMaterials* VKMaterials::GetInstance(){
	if (instance ==0){
		instance = new VKMaterials();
	}
	return instance;
}

/**
 * Searches for the material in the material manager. If the material is not
 * found in the material manager or can not be built, and exception is issued.
 *
 * @param material Material Name
 * @return A pointer to the material
 */
G4Material* VKMaterials::GetMaterial(const G4String material){

	// Trying to build the material
	G4Material* mat = nistMan->FindOrBuildMaterial(material);
	if (!mat) mat = G4Material::GetMaterial(material);

	// Exceptional error if it isn't found
	if(!mat) {
		std::ostringstream o;
		o<<"Material "<<material<<" not found.";
		G4Exception("Materials::GetMaterial","",FatalException,o.str().c_str());
	}
	return mat;
}


void VKMaterials::CreateMaterials(){

    // Elements
    fH  = new G4Element ("Hydrogen", "H" ,  1.,   1.01 *g/mole);
    fC  = new G4Element ("Carbon"  , "C" ,  6.,  12.01 *g/mole);
    fN  = new G4Element ("Nitrogen", "N" ,  7.,  14.01 *g/mole);
    fO  = new G4Element ("Oxygen"  , "O" ,  8.,  16.00 *g/mole);
    fAl = new G4Element ("Aluminium","Al", 13.,  26.98 *g/mole);
    fSi = new G4Element ("Silicon",  "Si", 14.,  28.09 *g/mole);
    fAr = new G4Element ("Argon"   , "Ar", 18.,  39.948*g/mole);
    fFe = new G4Element ("Iron"    , "Fe", 26.,  55.84 *g/mole);  
    fCu = new G4Element ("Copper"  , "Cu", 29.,  63.55 *g/mole);
    fZn = new G4Element ("Zinc"    , "Zn", 30.,  65.38 *g/mole);
    fBr = new G4Element ("Bromine",  "Br", 35.,  79.90 *g/mole);
    fLa = new G4Element ("Lanthanum","La", 57., 138.91 *g/mole);
    fCe = new G4Element ("Cerium",   "Ce", 58., 140.11 *g/mole);
    fPb = new G4Element ("Lead"    , "Pb", 82., 207.2  *g/mole);

    // Materials

    G4double density; 
    G4int components; 
    G4double temperature;
    G4double pressure;
    G4String name;
    G4double fractionalMass;

    // Argon
    fArgon = new G4Material(name="ARGON", density=1.7836*mg/cm3, components=1);
    fArgon->AddElement(fAr, 1);

    // Nitrogen
    fNitrogen = new G4Material(name="NITROGEN", density=1.25053*mg/cm3, components=1);
    fNitrogen->AddElement(fN, 2);

    // Oxygen
    fOxygen = new G4Material(name="OXYGEN", density=1.4289*mg/cm3, components=1);
    fOxygen->AddElement(fO, 2);

    // Laboratory vacuum (dry air)
    density  = 1.2928*mg/cm3 ;       // STP
    density *= 1.0e-9;               // pumped vacuum
    temperature = STP_Temperature;
    pressure = 1.0e-9*STP_Pressure;
    fLabVacuum = new G4Material(name="LABVACUUM", density, components=3, kStateGas,
                                temperature, pressure);
    fLabVacuum->AddMaterial(fNitrogen, fractionalMass=0.7557);
    fLabVacuum->AddMaterial(fOxygen,   fractionalMass=0.2315);
    fLabVacuum->AddMaterial(fArgon,    fractionalMass=0.0128);

    // Copper
    fCopper = new G4Material(name="COPPER", density=8.96*g/cm3, components=1);
    fCopper -> AddElement(fCu, 1);

    // Iron
    fIron = new G4Material(name="IRON", density=7.87*g/cm3, components=1);
    fIron -> AddElement(fFe, 1);

    // Brass (assumed to be Cu/Zn 1:1 by mass, density is only approximate)
    fBrass = new G4Material(name="BRASS", density=8.6*g/cm3, components=2);
    fBrass -> AddElement( fCu,  fractionalMass = 0.50 ) ;
    fBrass -> AddElement( fZn,  fractionalMass = 0.50 ) ;

    // Carbon (graphite)
    fCarbon = new G4Material(name="CARBON", density=2.267*g/cm3, components=1);
    fCarbon -> AddElement(fC, 1);
    
    // Aluminium
    fAluminium = new G4Material(name="ALUMINIUM", density=2.70*g/cm3, components=1);
    fAluminium -> AddElement(fAl, 1);
        
  	// EJ-200 Scintillator
    fEJ200 = nistMan -> FindOrBuildMaterial("G4_PLASTIC_SC_VINYLTOLUENE");

    // PMMA
    fPMMA = nistMan->FindOrBuildMaterial("G4_PLEXIGLASS");
    
    // Teflon (reflector)
    fTeflon = nistMan->FindOrBuildMaterial("G4_TEFLON");

    // Glass
    fGlass = new G4Material("GLASS", density=1.032*g/cm3, components=2);
    fGlass -> AddElement(fC, 91.533*perCent);
    fGlass -> AddElement(fH, 8.467*perCent);
}


void VKMaterials::SetOpticalPropertiesEJ200()
{

	// Index of refraction (146 nm to 1570 nm)
	const G4int nRInd = 2;
	G4double photonEnergyRInd[nRInd] = {8.5506*eV, 0.7973*eV};
	G4double refractiveIndex[nRInd]= {1.58, 1.58};

	// Absorbition Length
	const G4int nAbs = 2;
	G4double photonEnergyAbs[nAbs] = {3.5*eV, 1.75*eV};
	G4double absorptionLength[nAbs] = {380.0*cm, 380.0*cm};

	// Emission spectrum
	// See graph in: http://fcomp.ist.utl.pt/LRC.web/equipamento/EJ200-datasheet.pdf
	// Photon energy range: 3.01-2.49 eV
	// Equivalent wave-length range: 412-498 nm
	const G4int nEm = 6;
	G4double photonEnergyEm[nEm] = {3.01*eV, 2.92*eV, 2.82*eV, 2.70*eV, 2.58*eV, 2.49*eV};
	G4double emissionSpectrum[nEm] = {0, 1.0, 0.75, 0.45, 0.18, 0.05};

	// Creating the materials property table and adding entries
	G4MaterialPropertiesTable* EJ200_mpt = new G4MaterialPropertiesTable();
	
	EJ200_mpt -> AddProperty("RINDEX", photonEnergyRInd, refractiveIndex, nRInd);
	EJ200_mpt -> AddProperty("ABSLENGTH", photonEnergyAbs, absorptionLength, nAbs);
	EJ200_mpt -> AddProperty("FASTCOMPONENT", photonEnergyEm, emissionSpectrum, nEm)->SetSpline(true);
	EJ200_mpt -> AddProperty("SLOWCOMPONENT", photonEnergyEm, emissionSpectrum, nEm)->SetSpline(true);
    EJ200_mpt -> AddConstProperty("SCINTILLATIONYIELD", 10000/MeV);
    EJ200_mpt -> AddConstProperty("RESOLUTIONSCALE", 1.0);
    EJ200_mpt -> AddConstProperty("FASTTIMECONSTANT", 2.1*ns);
    EJ200_mpt -> AddConstProperty("SLOWTIMECONSTANT", 2.1*ns);
    EJ200_mpt -> AddConstProperty("SLOWSCINTILLATIONRISETIME", 0.9*ns);
    EJ200_mpt -> AddConstProperty("FASTSCINTILLATIONRISETIME", 0.9*ns);
    EJ200_mpt -> AddConstProperty("YIELDRATIO", 1.0);
	fEJ200 -> SetMaterialPropertiesTable(EJ200_mpt);

    // Birk's constant
    fEJ200 -> GetIonisation() -> SetBirksConstant(0.16*mm/MeV);
}


void VKMaterials::SetOpticalPropertiesPMMA(){

    // Index of refraction
	const G4int nRInd = 6;
	G4double photonEnergyRInd[nRInd] = 
	    {2.8450*eV, 2.5506*eV, 2.1100*eV, 1.7636*eV, 1.4884*eV, 1.1786*eV};
	G4double refractiveIndex[nRInd] =
	    {1.502, 1.497, 1.491, 1.486, 1.484, 1.481}; 

	// Absorbtion Length
	const G4int nAbs=3;
	G4double photonEnergyAbs[nAbs] = {3.3*eV, 4.12*eV, 2.75*eV};
	G4double absorptionLength[nAbs] = {0.2*m, 3*cm, 5*cm};

	// Add entries into properties table
	G4MaterialPropertiesTable* PMMA_mpt = new G4MaterialPropertiesTable();
	PMMA_mpt -> AddProperty("RINDEX", photonEnergyRInd, refractiveIndex, nRInd);
	PMMA_mpt -> AddProperty("ABSLENGTH", photonEnergyAbs, absorptionLength, nAbs);
	fPMMA -> SetMaterialPropertiesTable(PMMA_mpt);
}


void VKMaterials::SetOpticalPropertiesTeflon(){

	// Index of Reflection (146 nm to 1570 nm)
	const G4int nRInd = 13;
	G4double photonEnergyRInd[nRInd] = 
	    {8.5506*eV, 4.7232*eV, 3.2627*eV, 2.4921*eV, 2.0160*eV,
		 1.6926*eV, 1.4586*eV, 1.2815*eV, 1.1427*eV, 1.0311*eV, 
		 0.9393*eV, 0.8625*eV, 0.7973*eV};
	G4double refractiveIndex[nRInd] =
	    {1.4300, 1.3300, 1.3150, 1.3100, 1.3050,    
		 1.3050, 1.3000, 1.2900, 1.2975, 1.2970,
		 1.2960, 1.2950, 1.2950};

	// Absorbtion length (619 to 357 nm)
	const G4int nAbs = 4;
	G4double photonEnergyAbs[nAbs] = {2.00*eV, 2.87*eV, 2.90*eV, 3.47*eV};
	G4double absorptionLength[nAbs] = {9.00*m, 9.00*m, 0.1*mm, 0.1*mm};

	// Add entries into properties table
	G4MaterialPropertiesTable* Teflon_mpt = new G4MaterialPropertiesTable();
	Teflon_mpt -> AddProperty("RINDEX", photonEnergyRInd, refractiveIndex, nRInd);
	Teflon_mpt -> AddProperty("ABSLENGTH", photonEnergyAbs, absorptionLength, nAbs);
	fTeflon -> SetMaterialPropertiesTable(Teflon_mpt);
}


void VKMaterials::SetOpticalPropertiesGlass()
{
    // Index of reflection
	const G4int nRInd = 3;
	G4double photonEnergyRInd[nRInd] = {7.0*eV ,7.07*eV, 7.14*eV};
    G4double refractiveIndex[nRInd] = {1.49, 1.49, 1.49};

    // Absorption length
	const G4int nAbs = 3;
	G4double photonEnergyAbs[nAbs] = {7.0*eV ,7.07*eV, 7.14*eV};
    G4double absorptionLength[nAbs] = {420.*cm, 420.*cm, 420.*cm};

    // properties table
    G4MaterialPropertiesTable *Glass_mpt = new G4MaterialPropertiesTable();
    Glass_mpt -> AddProperty("RINDEX", photonEnergyRInd, refractiveIndex, nRInd);
    Glass_mpt -> AddProperty("ABSLENGTH", photonEnergyAbs, absorptionLength, nAbs);
    fGlass -> SetMaterialPropertiesTable(Glass_mpt);
}
