
#include <unittest++/UnitTest++.h>

#include "G4RunManager.hh"
#include "G4UImanager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SystemOfUnits.hh"

#include "VKDetectorConstruction.hh"


SUITE(VKDetectorTest) {

    class SetupFixture {
        public:
            SetupFixture()
            {
                // get pointer to detector
                detector = static_cast<const VKDetectorConstruction*>
                    (G4RunManager::GetRunManager()->GetUserDetectorConstruction());
                
                // get pointer to UI manager
                UImanager = G4UImanager::GetUIpointer();

                // get pointer to physical volumes store
                physVolStore = 
                    static_cast<const G4PhysicalVolumeStore*>(G4PhysicalVolumeStore::GetInstance());

                // get pointer to logical volumes store
                logVolStore = 
                    static_cast<const G4LogicalVolumeStore*>(G4LogicalVolumeStore::GetInstance());
            }
            const VKDetectorConstruction *detector;
            G4UImanager *UImanager;
            const G4PhysicalVolumeStore *physVolStore;
            const G4LogicalVolumeStore *logVolStore;
    };

    TEST_FIXTURE(SetupFixture, Sanity) {
        CHECK_EQUAL(1, 1);
    }
    
    TEST_FIXTURE(SetupFixture, TestMagnetOnMessenger) {
        UImanager->ApplyCommand("/VK/detector/volumes/magnet false");
        bool status = detector->GetMagnetOn();
        CHECK_EQUAL(false, status);
        UImanager->ApplyCommand("/VK/detector/volumes/magnet true");
        bool newStatus = detector->GetMagnetOn();
        CHECK_EQUAL(true, newStatus);
    }    

    TEST_FIXTURE(SetupFixture, TestWorldVolumeExists) {
        bool worldExists = physVolStore->GetVolume("world") != nullptr;
        CHECK_EQUAL(true, worldExists);
    }

    TEST_FIXTURE(SetupFixture, TestCoilsExist) {
        UImanager->ApplyCommand("/VK/detector/volumes/magnet true");
        G4bool verbose = false;
        bool physVolExists = physVolStore->GetVolume("coils", verbose) != nullptr;
        CHECK_EQUAL(true, physVolExists);
    }

    TEST_FIXTURE(SetupFixture, TestIronWallsExist) {
        UImanager->ApplyCommand("/VK/detector/volumes/magnet true");
        G4bool verbose = false;
        bool physVolExists = physVolStore->GetVolume("ironWalls", verbose) != nullptr;
        CHECK_EQUAL(true, physVolExists);
        bool logVolExists = logVolStore->GetVolume("ironWalls_log", verbose) != nullptr;
        CHECK_EQUAL(true, logVolExists);
    }

    TEST_FIXTURE(SetupFixture, TestMagnetSuccessfullyRemoved) {
        UImanager->ApplyCommand("/VK/detector/volumes/magnet false");
        G4bool verbose = false;
        G4bool physVolExists = physVolStore->GetVolume("coils", verbose) != nullptr;
        CHECK_EQUAL(false, physVolExists);
        CHECK_EQUAL(false, physVolExists);
    }

    TEST_FIXTURE(SetupFixture, TestSignalExists) {
        UImanager->ApplyCommand("/VK/detector/volumes/betaDetector true");
        G4bool verbose = false;
        bool physVolExists = physVolStore->GetVolume("signal", verbose) != nullptr;
        CHECK_EQUAL(true, physVolExists);
        bool logVolExists = logVolStore->GetVolume("signal_log", verbose) != nullptr;
        CHECK_EQUAL(true, logVolExists);
    }

    TEST_FIXTURE(SetupFixture, TestVetoExists) {
        UImanager->ApplyCommand("/VK/detector/volumes/betaDetector true");
        G4bool verbose = false;
        bool physVolExists = physVolStore->GetVolume("veto", verbose) != nullptr;
        CHECK_EQUAL(true, physVolExists);
        bool logVolExists = logVolStore->GetVolume("veto_log", verbose) != nullptr;
        CHECK_EQUAL(true, logVolExists);
    }

    TEST_FIXTURE(SetupFixture, TestWindowExists) {
        UImanager->ApplyCommand("/VK/detector/volumes/betaDetector true");
        G4bool verbose = false;
        bool physVolExists = physVolStore->GetVolume("window", verbose) != nullptr;
        CHECK_EQUAL(true, physVolExists);
        bool logVolExists = logVolStore->GetVolume("window_log", verbose) != nullptr;
        CHECK_EQUAL(true, logVolExists);
    }

    TEST_FIXTURE(SetupFixture, TestSignalSuccessfullyRemoved) {
        UImanager->ApplyCommand("/VK/detector/volumes/betaDetector false");
        G4bool verbose = false;
        G4bool signalExists = physVolStore->GetVolume("signal", verbose) != nullptr;
        CHECK_EQUAL(false, signalExists);
    }

    TEST_FIXTURE(SetupFixture, TestCatcherFoilSuccessfullyRemoved) {
        UImanager->ApplyCommand("/VK/detector/volumes/catcherFoil false");
        G4bool verbose = false;
        G4bool physVolExists = physVolStore->GetVolume("foil", verbose) != nullptr;
        CHECK_EQUAL(false, physVolExists);
        physVolExists = physVolStore->GetVolume("frame", verbose) != nullptr;
        CHECK_EQUAL(false, physVolExists);
    }

    TEST_FIXTURE(SetupFixture, TestShieldAluFrameSuccessfullyRemoved) {
        UImanager->ApplyCommand("/VK/detector/volumes/shield false");
        G4bool verbose = false;
        G4bool physVolExists = physVolStore->GetVolume("shieldAluFrame", verbose) != nullptr;
        CHECK_EQUAL(false, physVolExists);
    }

    TEST_FIXTURE(SetupFixture, TestVirtualDetectorsSuccessfullyRemoved) {
        G4bool verbose = false;
        UImanager->ApplyCommand("/VK/storeTrajectoryData true");
        G4bool physVolExists = physVolStore->GetVolume("focalPlane", verbose) != nullptr;
        CHECK_EQUAL(true, physVolExists);
        UImanager->ApplyCommand("/VK/storeTrajectoryData false");
        physVolExists = physVolStore->GetVolume("focalPlane", verbose) != nullptr;
        CHECK_EQUAL(false, physVolExists);
    }

}
